@@ -138,7 +138,7 @@ fn convert_to_xlsx(csv_path: &str) -> PyResult<()> {
             return Err(PyValueError::new_err(format!(
                 "Failed to open CSV file: {}",
                 e
-            )))
+            )));
         }
     };
     for (row_number, row) in csv_reader.records().enumerate() {
@@ -148,7 +148,7 @@ fn convert_to_xlsx(csv_path: &str) -> PyResult<()> {
                 return Err(PyValueError::new_err(format!(
                     "Failed to read CSV row: {}",
                     e
-                )))
+                )));
             }
         };
         for (column, cell) in row_data.iter().enumerate() {
@@ -204,6 +204,10 @@ struct DictWriter {
     file_object: Py<PyAny>,
     // The header fields, in order.
     fieldnames: Vec<String>,
+    // The delimiter character for CSV output (default: comma)
+    delimiter: u8,
+    // How to handle extra fields: "raise" or "ignore" (default: "raise")
+    extrasaction: String,
     // The internal buffer for CSV data
     buffer: Vec<u8>,
     // Buffer for reusing string vectors to reduce allocations
@@ -213,7 +217,14 @@ struct DictWriter {
 #[pymethods]
 impl DictWriter {
     #[new]
-    fn __new__(py: Python, f: Py<PyAny>, fieldnames: Vec<String>) -> PyResult<Self> {
+    #[pyo3(signature = (f, fieldnames, delimiter=",", extrasaction="raise"))]
+    fn __new__(
+        py: Python,
+        f: Py<PyAny>,
+        fieldnames: Vec<String>,
+        delimiter: &str,
+        extrasaction: &str,
+    ) -> PyResult<Self> {
         // Ensure the file-like object has a 'write' method.
         let file_obj = f.bind(py);
         if !file_obj.hasattr("write")? {
@@ -222,6 +233,21 @@ impl DictWriter {
             ));
         }
 
+        // Validate delimiter (should be exactly one character)
+        if delimiter.len() != 1 {
+            return Err(PyValueError::new_err(
+                "delimiter must be a single character",
+            ));
+        }
+        let delimiter_byte = delimiter.as_bytes()[0];
+
+        // Validate extrasaction
+        if extrasaction != "raise" && extrasaction != "ignore" {
+            return Err(PyValueError::new_err(
+                "extrasaction must be 'raise' or 'ignore'",
+            ));
+        }
+
         // Create a CSV writer that writes to an internal, growable byte buffer.
         // Use standard LF line endings (\n) which is modern practice on Unix systems
         let buffer = Vec::new();
@@ -229,6 +255,8 @@ impl DictWriter {
         Ok(DictWriter {
             file_object: f,
             fieldnames: fieldnames.clone(),
+            delimiter: delimiter_byte,
+            extrasaction: extrasaction.to_string(),
             buffer,
             string_buffer: Vec::with_capacity(fieldnames.len()),
         })
@@ -236,8 +264,10 @@ impl DictWriter {
 
     /// Writes the header row to the file.
     pub fn writeheader(&mut self, py: Python) -> PyResult<()> {
-        // Create a temporary writer for this operation
-        let mut writer = csv::WriterBuilder::new().from_writer(&mut self.buffer);
+        // Create a temporary writer for this operation with custom delimiter
+        let mut writer = csv::WriterBuilder::new()
+            .delimiter(self.delimiter)
+            .from_writer(&mut self.buffer);
         writer
             .write_record(&self.fieldnames)
             .map_err(csv_error_to_py_err)?;
@@ -255,6 +285,20 @@ impl DictWriter {
         self.string_buffer.clear();
         self.string_buffer.reserve(self.fieldnames.len());
 
+        // Check for extra keys if extrasaction is "raise"
+        if self.extrasaction == "raise" {
+            for key in row.keys() {
+                let key_bound = key.str()?;
+                let key_str = key_bound.to_str()?;
+                if !self.fieldnames.contains(&key_str.to_string()) {
+                    return Err(PyValueError::new_err(format!(
+                        "dict contains fields not in fieldnames: '{}'",
+                        key_str
+                    )));
+                }
+            }
+        }
+
         for key in &self.fieldnames {
             let value = match row.get_item(key) {
                 Ok(Some(val)) => val.str()?.to_str()?.to_string(),
@@ -263,8 +307,10 @@ impl DictWriter {
             self.string_buffer.push(value);
         }
 
-        // Create a temporary writer for this operation
-        let mut writer = csv::WriterBuilder::new().from_writer(&mut self.buffer);
+        // Create a temporary writer for this operation with custom delimiter
+        let mut writer = csv::WriterBuilder::new()
+            .delimiter(self.delimiter)
+            .from_writer(&mut self.buffer);
         writer
             .write_record(&self.string_buffer)
             .map_err(csv_error_to_py_err)?;
@@ -279,11 +325,27 @@ impl DictWriter {
     /// Writes a list of dictionary rows to the file.
     /// Optimized to batch process rows and reduce Python interop overhead.
     pub fn writerows(&mut self, py: Python, rows: Vec<Bound<PyDict>>) -> PyResult<()> {
-        // Create a single writer for all rows to maximize efficiency
-        let mut writer = csv::WriterBuilder::new().from_writer(&mut self.buffer);
+        // Create a single writer for all rows to maximize efficiency with custom delimiter
+        let mut writer = csv::WriterBuilder::new()
+            .delimiter(self.delimiter)
+            .from_writer(&mut self.buffer);
 
         // Process all rows first, then flush once at the end
         for row in &rows {
+            // Check for extra keys if extrasaction is "raise"
+            if self.extrasaction == "raise" {
+                for key in row.keys() {
+                    let key_bound = key.str()?;
+                    let key_str = key_bound.to_str()?;
+                    if !self.fieldnames.contains(&key_str.to_string()) {
+                        return Err(PyValueError::new_err(format!(
+                            "dict contains fields not in fieldnames: '{}'",
+                            key_str
+                        )));
+                    }
+                }
+            }
+
             // Reuse the string buffer to avoid allocations
             self.string_buffer.clear();
             self.string_buffer.reserve(self.fieldnames.len());
